@@ -5,192 +5,47 @@
 import logging
 log = logging.getLogger(__name__)
 
-from dsdobjects.parser import (parse_pil_string, 
-                               parse_pil_file,
-                               parse_seesaw_string,
-                               parse_seesaw_file,
-                               ParseException, 
-                               PilFormatError)
-from peppercornenumerator.objects import (PepperDomain, 
-                                          PepperComplex, 
-                                          PepperReaction, 
-                                          DSDDuplicationError)
+from dsdobjects.objectio import set_io_objects, clear_io_objects
+from dsdobjects.objectio import read_pil as dsd_read_pil
+from dsdobjects.objectio import read_pil_line as dsd_read_pil_line
+from dsdobjects import (parse_pil_string,
+                        parse_seesaw_string,
+                        parse_seesaw_file)
+from .objects import (PepperDomain, 
+                      PepperStrand,
+                      PepperComplex, 
+                      PepperReaction, 
+                      PepperMacrostate)
+
 
 class InputFormatError(Exception):
     pass
 
-def resolve_loops(loop):
-    """ Return a sequence, structure pair from kernel format with parenthesis. """
-    sequen = []
-    struct = []
-    for dom in loop :
-        if isinstance(dom, str):
-            sequen.append(dom)
-            if dom == '+' :
-                struct.append('+')
-            else :
-                struct.append('.')
-        elif isinstance(dom, list):
-            struct[-1] = '('
-            old = sequen[-1]
-            se, ss = resolve_loops(dom)
-            sequen.extend(se)
-            struct.extend(ss)
-            sequen.append(old + '*' if old[-1] != '*' else old[:-1])
-            struct.append(')')
-    return sequen, struct
+class PilFormatError(Exception):
+    pass
 
-def read_reaction(line):
-    rtype = line[1][0][0] if line[1] != [] and line[1][0] != [] else None
-    rate = float(line[1][1][0]) if line[1] != [] and line[1][1] != [] else None
-    error = float(line[1][1][1]) if line[1] != [] and line[1][1] != [] and len(line[1][1]) == 2 else None
-    units = line[1][2][0] if line[1] != [] and line[1][2] != [] else None
+def read_pil_line(*args, **kwargs):
+    set_io_objects(D = PepperDomain, S = PepperStrand, C = PepperComplex, R = PepperReaction, M = PepperMacrostate)
+    out = dsd_read_pil_line(*args, **kwargs)
+    clear_io_objects()
+    return out
 
-    if rate is None:
-        r = "{} -> {}".format(' + '.join(line[2]), ' + '.join(line[3]))
-        log.warning("Ignoring input reaction without a rate: {}".format(r))
-        return None, None, None, None, None, None
-    elif rtype is None or rtype == 'condensed' or rtype not in PepperReaction.RTYPES:
-        r = "{} -> {}".format(' + '.join(line[2]), ' + '.join(line[3]))
-        log.warning("Ignoring input reaction of with rtype='{}': {}".format(rtype, r))
-        return None, None, None, None, None, None
-    else :
-        r = "[{} = {:12g} {}] {} -> {}".format(
-                rtype, rate, units, ' + '.join(line[2]), ' + '.join(line[3]))
+def read_pil(data, is_file = False, ignore = None):
+    set_io_objects(D = PepperDomain, S = PepperStrand, C = PepperComplex, R = PepperReaction, M = PepperMacrostate)
+    out = dsd_read_pil(data, is_file, ignore)
+    clear_io_objects()
 
-    return line[2], line[3], rtype, rate, units, r
+    if len(out['con_reactions']):
+        log.warning('Ignoring condensed reaction input.')
+    if len(out['macrostates']):
+        log.warning('Ignoring complex macrostate input.')
 
-def read_pil(data, is_file = False, composite = False):
-    """ Peppercorn standard input.
-
-    Supports a variety of pil-style dialects, including kernel and enum.
-
-    Args:
-        data (str): Is either the PIL file in string format or the path to a file.
-        is_file (bool): True if data is a path to a file, False otherwise
-        composite (bool, optional): Returns an additional dictionary that maps
-            names of composite domains (or strands) to a list of domains.
-    """
-    if is_file :
-        parsed_file = parse_pil_file(data)
-    else :
-        parsed_file = parse_pil_string(data)
-
-    domains = {'+' : '+'} # saves some code
-    sequences = {}
-    complexes = {}
-    reactions = []
-    for line in parsed_file :
-        name = line[1]
-        if line[0] == 'dl-domain':
-            if line[2] == 'short':
-                (dtype, dlen) = ('short', None)
-            elif line[2] == 'long':
-                (dtype, dlen) = ('long', None)
-            else :
-                (dtype, dlen) = (None, int(line[2]))
-            if name not in domains:
-                domains[name] = PepperDomain(name, dtype = dtype, length = dlen)
-            cname = name[:-1] if domains[name].is_complement else name + '*'
-            if cname in domains:
-                assert domains[cname] == ~domains[name]
-            else :
-                domains[cname] = ~domains[name]
-
-        elif line[0] == 'sl-domain':
-            log.info("Ignoring sequence information for domain {}.".format(name))
-            if len(line) == 4:
-                if int(line[3]) != len(line[2]):
-                    raise InputFormatError("Sequence/Length information inconsistent {} vs {}.".format(line[3], len(line[2])))
-                domains[name] = PepperDomain(name, length = int(line[3]))
-            else :
-                domains[name] = PepperDomain(name, length = len(line[2]))
-
-            domains[name].nucleotides = line[2]
-            cname = name[:-1] if domains[name].is_complement else name + '*'
-            if cname in domains:
-                assert domains[cname] == ~domains[name]
-            else :
-                domains[cname] = ~domains[name]
-
-        elif line[0] == 'composite-domain':
-            # This could be a strand definition or a composite domain.
-            assert name[-1] != '*'
-            sequences[name] = list([domains[x] for x in line[2]])
-
-            def comp(name):
-                return name[:-1] if name[-1] == '*' else name + '*'
-
-            sequences[comp(name)] = list([domains[comp(x)] for x in reversed(line[2])])
-
-        elif line[0] == 'strand-complex':
-            sequence = []
-            for strand in line[2]:
-                sequence += sequences[strand] + ['+']
-            sequence = sequence[:-1]
-            structure = line[3].replace(' ','')
-            complexes[name] = PepperComplex(sequence, list(structure), name=name)
-
-        elif line[0] == 'kernel-complex':
-            sequence, structure = resolve_loops(line[2])
-
-            # Replace names with domain objects.
-            try :
-                sequence = list([domains[d] for d in sequence])
-            except KeyError:
-                for e, d in enumerate(sequence):
-                    if isinstance(d, PepperDomain):
-                        # Happens with composite domains, see next statement e+1
-                        continue
-                    if d in sequences :
-                        for i, c in enumerate(sequences[d]):
-                            assert c.name in domains
-                            if i == 0:
-                                sequence[e] = c
-                            else :
-                                sequence.insert(e+i, c)
-                                structure.insert(e+i, structure[e])
-
-                    elif d not in domains :
-                        log.warning("Assuming {} is a long domain.".format(d))
-                        domains[d] = PepperDomain(d, 'long')
-                        cdom = ~domains[d]
-                        domains[cdom.name] = cdom
-                        sequence[e] = domains[d]
-                    else :
-                        sequence[e] = domains[d]
-
-            complexes[name] = PepperComplex(sequence, structure, name=name)
-
-            if len(line) > 3 :
-                assert len(line[3]) == 3
-                complexes[name].concentration = tuple([line[3][0], float(line[3][1]), line[3][2]])
-
-
-        elif line[0] == 'reaction':
-            reactants, products, rtype, rate, units, r = read_reaction(line)
-            if r is None: continue
-
-            try :
-                reactants = list([complexes[c] for c in reactants])
-                products  = list([complexes[c] for c in products])
-            except KeyError:
-                log.warning("Ignoring input reaction with undefined complex: {}".format(r))
-                continue
-            
-            reaction = PepperReaction(reactants, products, rtype=rtype, rate=rate)
-            if reaction.rateunits != units:
-                log.error("Rate units must be given in {}, not: {}.".format(reaction.rateunits, units))
-                raise SystemExit
-            reactions.append(reaction)
-
-        else :
-            log.warning("Ignoring {} specification: {}".format(line[0], name))
-        
-    if composite :
-        return complexes, reactions, sequences
-    else :
-        return complexes, reactions
+    # Make sure that strands are not within initial complexes!
+    for strand in out['strands'].values():
+        strand.concentration = ('constant', 0, 'M')
+        out['complexes'][('s', strand.name)] = strand
+    out['domains'].clear()
+    return out['complexes'], out['det_reactions']
 
 def load_pil_crn(data):
     """ Input for pilsimulator
@@ -308,8 +163,8 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
         tlen =  5 # lenth of the threshold
 
     complexes = {}
-    def make_wire(w, name=None):
-        """Translates a wire into a PepperComplex.
+    def make_wire(w, name = None):
+        """ Translates a wire into a PepperComplex.
 
         Every complex gets 0 M concentration as default, i.e. it is excluded
         from enumeration.
@@ -330,10 +185,10 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
         [nI, nO] = w[1]
         wire = 'w{}_{}'.format(nI, nO)
         if wire not in complexes:
-            dIa = assgn_domain('s{}a'.format(nI), l=dlen)
-            dIt = assgn_domain('s{}t'.format(nI), l=tlen)
-            dOa = assgn_domain('s{}a'.format(nO), l=dlen)
-            dOt = assgn_domain('s{}t'.format(nO), l=tlen)
+            dIa = assgn_domain(f's{nI}a', l=dlen)
+            dIt = assgn_domain(f's{nI}t', l=tlen)
+            dOa = assgn_domain(f's{nO}a', l=dlen)
+            dOt = assgn_domain(f's{nO}t', l=tlen)
             if explicit:
                 sequence = [clp, dOt, dOa, clp, toe, clp, dIt, dIa, clp]
                 structure = list('.........')
@@ -341,7 +196,7 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
                 sequence = [dOt, dOa, toe, dIt, dIa]
                 structure = list('.....')
             complexes[wire] = PepperComplex(sequence, structure, 
-                    name=name if name else wire)
+                    name = name if name else wire)
             complexes[wire].concentration = ('i', 0, 'M')
         return complexes[wire]
 
@@ -445,14 +300,14 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
 
     for line in parsed_file :
         if line[0] == 'INPUT':
-            # use the given name if it is not a digit...
+            # use the given name if it is not a digit ...
             name = None if line[1][0].isdigit() else line[1][0]
             w = make_wire(line[2], name)
-            assert w.concentration.value == 0
+            assert w.concentration[1] == 0
             w.concentration = None
 
         elif line[0] == 'OUTPUT':
-            # use the given name if it is not a digit...
+            # use the given name if it is not a digit ...
             name = None if line[1][0].isdigit() else line[1][0]
             if line[2][0] == 'w':
                 make_wire(line[2], name)
@@ -507,15 +362,15 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
             Iw = make_wire(['w',[g1, g2]])
             Th,w1,w2 = make_thld(['t',[g1, g2]]) 
             if line[0] == 'seesawOR':
-                assert Th.concentration.value == 0
+                assert Th.concentration[1] == 0
                 Th.concentration = ('i', 1.1 * 0.6 * conc, 'M')
             else:
-                assert Th.concentration.value == 0
+                assert Th.concentration[1] == 0
                 Th.concentration = ('i', 1.1 * (n-1 + 0.2) * conc, 'M')
 
             # Connection reaction (assign intial concentration)
             Ig = make_gate(['g', [g1, ['w', [g1, g2]]]])
-            assert Ig.concentration.value == 0
+            assert Ig.concentration[1] == 0
             Ig.concentration = ('i', n * conc, 'M')
 
             # fanout for all outputs
@@ -527,10 +382,10 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
                 Og = make_gate(['g', [['w', [g1, g2]], g2]]) #consumed gate
 
                 if d == 'f':
-                    assert Ow.concentration.value == 0
-                    fmod = Ow.concentration.mode
-                    fcon = Ow.concentration.value
-                    funi = Ow.concentration.unit
+                    assert Ow.concentration[1] == 0
+                    fmod = Ow.concentration[0]
+                    fcon = Ow.concentration[1]
+                    funi = Ow.concentration[2]
                     fcon += 2 * m * conc
                     Ow.concentration = (fmod, fcon, funi)
                 else :
@@ -543,7 +398,7 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
 
             make_wire(['w', [inp, g1]])
             cx,_,_ = make_thld(['t', [inp, g1]])
-            assert cx.concentration.value == 0
+            assert cx.concentration[1] == 0
             cx.concentration = ('i', 1.1 * 0.2 * conc, 'M')
             make_gate(['g', [['w',[inp, g1]], g1]])
 
@@ -551,14 +406,14 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
                 cxw = make_wire(['w', [g1, d]])
                 cxg = make_gate(['g', [g1, ['w', [g1, d]]]])
                 if d == 'f':
-                    assert cxw.concentration.value == 0
-                    cxwmod = Ow.concentration.mode
-                    cxwcon = Ow.concentration.value
-                    cxwuni = Ow.concentration.unit
+                    assert cxw.concentration[1] == 0
+                    cxwmod = Ow.concentration[0]
+                    cxwcon = Ow.concentration[1]
+                    cxwuni = Ow.concentration[2]
                     cxwcon += 2 * m * conc
                     cxw.concentration = (cxwmod, cxwcon, cxwuni)
                 else :
-                    assert cxg.concentration.value == 0
+                    assert cxg.concentration[1] == 0
                     cxg.concentration = ('i', conc, 'M')
 
         elif line[0] == 'reporter':
@@ -614,7 +469,7 @@ def read_seesaw(data, is_file = False, conc = 100e-9, explicit = True, reactions
             else:
                 raise NotImplementedError('Cannot interpret line {}'.format(line))
             # input wires have None at default
-            assert cx.concentration is None or cx.concentration.value == 0
+            assert cx.concentration is None or cx.concentration[1] == 0
             cx.concentration = ('i', conc*float(line[2]), 'M')
 
         else :
@@ -685,13 +540,12 @@ def get_seesaw_compiler_reactions(complexes, T = 20, utbr = False, leak = False,
         elif k[0]=='T':
             thlds.append(x)
 
-    count=0
+    count = 0
     reactions = []
-    
-    if utbr and global_species :
+    if utbr and global_species:
         # Make the universal wire W
         # This is the fuel value... 
-        conc = sum([x.concentration.value if x.concentration else 0 for x in wires])
+        conc = sum([x.concentration[1] if x.concentration else 0 for x in wires])
         if conc:
             uW = PepperDomain('dummy_{}'.format('W'), dtype='long')
             uW = PepperComplex(sequence=[uW], structure=['.'], name='W')
@@ -701,14 +555,14 @@ def get_seesaw_compiler_reactions(complexes, T = 20, utbr = False, leak = False,
         # Make the universal gate G (Gates and Reporters)
         uG = PepperDomain('dummy_{}'.format('G'), dtype='long')
         uG = PepperComplex(sequence=[uG], structure=['.'], name='G')
-        conc = sum([x.concentration.value if x.concentration else 0 for x in gates + freps])
+        conc = sum([x.concentration[1] if x.concentration else 0 for x in gates + freps])
         uG.concentration = ('initial', conc, 'M')
         complexes[uG.name] = uG
 
         # Make the universal threshold TH 
         uT = PepperDomain('dummy_{}'.format('TH'), dtype='long')
         uT = PepperComplex(sequence=[uT], structure=['.'], name='TH')
-        conc = sum([x.concentration.value if x.concentration else 0 for x in thlds])
+        conc = sum([x.concentration[1] if x.concentration else 0 for x in thlds])
         uT.concentration = ('initial', conc, 'M')
         complexes[uT.name] = uT
 
@@ -737,57 +591,77 @@ def get_seesaw_compiler_reactions(complexes, T = 20, utbr = False, leak = False,
             if G == '+' and wo == gn:
                 # a regular interaction
                 og = complexes['G_' + w.seesawname + '_' + 'g' + gn]
-                reactions.append(PepperReaction([w, g], [gw, og], rtype='seesaw', rate=ks))
+                rxn = PepperReaction([w, g], [gw, og], rtype = 'seesaw')
+                rxn.rate_constant = (ks, '/M/s')
+                reactions.append(rxn)
                 continue
 
             if G == '-' and wi == gn:
                 # a regular interaction
                 og = complexes['G_' + 'g' + gn + '_' + w.seesawname]
-                reactions.append(PepperReaction([w, g], [gw, og], rtype='seesaw', rate=ks))
+                rxn = PepperReaction([w, g], [gw, og], rtype = 'seesaw')
+                rxn.rate_constant = (ks, '/M/s')
+                reactions.append(rxn)
                 continue
 
             if leak and G == '+' and wi == gwi:# and wo != gwo:
                 og = complexes['G_' + 'g' + gn + '_' + w.seesawname]
-                reactions.append(PepperReaction([w, g], [gw, og], rtype='side-leak', rate=kl))
+                rxn = PepperReaction([w, g], [gw, og], rtype = 'side-leak')
+                rxn.rate_constant = (kl, '/M/s')
+                reactions.append(rxn)
 
             if leak and G == '-' and wo == gwo:# and wi != gwi:
                 og = complexes['G_' + w.seesawname + '_' + 'g' + gn]
-                reactions.append(PepperReaction([w, g], [gw, og], rtype='side-leak', rate=kl))
+                rxn = PepperReaction([w, g], [gw, og], rtype = 'side-leak')
+                rxn.rate_constant = (kl, '/M/s')
+                reactions.append(rxn)
 
             if utbr and global_species:
                 # TODO: check if that makes sense...?
-                if w.concentration and w.concentration.value != 0:
-                    try : # all Wires together
-                        gWn = g.name + '_W'
-                        gW = PepperDomain('dummy_{}'.format(gWn), dtype='long')
-                        gW = PepperComplex(sequence=[gW], structure=['.'], name=gWn)
-                        gW.concentration = ('initial', 0, 'M')
-                        complexes[gWn] = gW
+                if w.concentration and w.concentration[1] != 0:
+                    #try : # all Wires together
+                    gWn = g.name + '_W'
+                    gW = PepperDomain('dummy_{}'.format(gWn), dtype='long')
+                    gW = PepperComplex(sequence=[gW], structure=['.'], name=gWn)
+                    gW.concentration = ('initial', 0, 'M')
+                    complexes[gWn] = gW
 
-                        reactions.append(PepperReaction([g, uW], [gW], rtype='side-utbr', rate=kf))
-                        reactions.append(PepperReaction([gW], [g, uW], rtype='side-utbr', rate=krf))
-                    except DSDDuplicationError as err:
-                        pass
+                    rxn = PepperReaction([g, uW], [gW], rtype = 'side-utbr')
+                    rxn.rate_constant = (kf, '/M/s')
+                    reactions.append(rxn)
+                    rxn = PepperReaction([gW], [g, uW], rtype = 'side-utbr')
+                    rxn.rate_constant = (krf, '/s')
+                    reactions.append(rxn)
+                    #except DSDDuplicationError as err:
+                    #    pass
                 else :
-                    try: # all Gates together
-                        wGn = w.name + '_G'
-                        wG = PepperDomain('dummy_{}'.format(wGn), dtype='long')
-                        wG = PepperComplex(sequence=[wG], structure=['.'], name=wGn)
-                        wG.concentration = ('initial', 0, 'M')
-                        complexes[wGn] = wG
+                    #try: # all Gates together
+                    wGn = w.name + '_G'
+                    wG = PepperDomain('dummy_{}'.format(wGn), dtype='long')
+                    wG = PepperComplex(sequence=[wG], structure=['.'], name=wGn)
+                    wG.concentration = ('initial', 0, 'M')
+                    complexes[wGn] = wG
 
-                        reactions.append(PepperReaction([w, uG], [wG], rtype='side-utbr', rate=kf))
-                        reactions.append(PepperReaction([wG], [w, uG], rtype='side-utbr', rate=krf))
-                    except DSDDuplicationError as err:
-                        pass
+                    rxn = PepperReaction([w, uG], [wG], rtype = 'side-utbr')
+                    rxn.rate_constant = (kf, '/M/s')
+                    reactions.append(rxn)
+                    rxn = PepperReaction([wG], [w, uG], rtype = 'side-utbr')
+                    rxn.rate_constant = (krf, '/s')
+                    reactions.append(rxn)
+                    #except DSDDuplicationError as err:
+                    #    pass
 
             elif utbr :
                 count += 1
                 a = PepperDomain('dummy_{}'.format(count), dtype='long')
                 a = PepperComplex(sequence=[a], structure=['.'], prefix='s')
                 complexes[a.name] = a
-                reactions.append(PepperReaction([w, g], [a], rtype='side-utbr', rate=kf))
-                reactions.append(PepperReaction([a], [w, g], rtype='side-utbr', rate=krf))
+                rxn = PepperReaction([w, g], [a], rtype = 'side-utbr')
+                rxn.rate_constant = (kf, '/M/s')
+                reactions.append(rxn)
+                rxn = PepperReaction([a], [w, g], rtype = 'side-utbr')
+                rxn.rate_constant = (krf, '/s')
+                reactions.append(rxn)
         
         for g in freps:
             # R_6
@@ -795,85 +669,113 @@ def get_seesaw_compiler_reactions(complexes, T = 20, utbr = False, leak = False,
                 # a regular interaction
                 of = complexes['F_'+wo]
                 oq = complexes['Q_'+wo]
-                reactions.append(PepperReaction([w, g], [of, oq], rtype='seesaw-rep', rate=ks))
+                rxn = PepperReaction([w, g], [of, oq], rtype = 'seesaw-rep')
+                rxn.rate_constant = (ks, '/M/s')
+                reactions.append(rxn)
                 continue
 
             if utbr and global_species:
                 # TODO: check if that makes sense...?
-                if w.concentration and w.concentration.value != 0:
-                    try: # all-wires product
-                        gWn = g.name + '_W'
-                        gW = PepperDomain('dummy_{}'.format(gWn), dtype='long')
-                        gW = PepperComplex(sequence=[gW], structure=['.'], name=gWn)
-                        gW.concentration = ('initial', 0, 'M')
-                        complexes[gWn] = gW
+                if w.concentration and w.concentration[1] != 0:
+                    #try: # all-wires product
+                    gWn = g.name + '_W'
+                    gW = PepperDomain('dummy_{}'.format(gWn), dtype='long')
+                    gW = PepperComplex(sequence=[gW], structure=['.'], name=gWn)
+                    gW.concentration = ('initial', 0, 'M')
+                    complexes[gWn] = gW
 
-                        reactions.append(PepperReaction([g, uW], [gW], rtype='side-utbr', rate=kf))
-                        reactions.append(PepperReaction([gW], [g, uW], rtype='side-utbr', rate=krf))
-                    except DSDDuplicationError as err:
-                        pass
+                    rxn = PepperReaction([g, uW], [gW], rtype = 'side-utbr')
+                    rxn.rate_constant = (kf, '/M/s')
+                    reactions.append(rxn)
+                    rxn = PepperReaction([gW], [g, uW], rtype = 'side-utbr')
+                    rxn.rate_constant = (krf, '/s')
+                    reactions.append(rxn)
+                    #except DSDDuplicationError as err:
+                    #    pass
                 else:
-                    try: # all Gates together
-                        wGn = w.name + '_G'
-                        wG = PepperDomain('dummy_{}'.format(wGn), dtype='long')
-                        wG = PepperComplex(sequence=[wG], structure=['.'], name=wGn)
-                        wG.concentration = ('initial', 0, 'M')
-                        complexes[wGn] = wG
+                    #try: # all Gates together
+                    wGn = w.name + '_G'
+                    wG = PepperDomain('dummy_{}'.format(wGn), dtype='long')
+                    wG = PepperComplex(sequence=[wG], structure=['.'], name=wGn)
+                    wG.concentration = ('initial', 0, 'M')
+                    complexes[wGn] = wG
 
-                        reactions.append(PepperReaction([w, uG], [wG], rtype='side-utbr', rate=kf))
-                        reactions.append(PepperReaction([wG], [w, uG], rtype='side-utbr', rate=krf))
-                    except DSDDuplicationError as err:
-                        pass
+                    rxn = PepperReaction([w, uG], [wG], rtype = 'side-utbr')
+                    rxn.rate_constant = (kf, '/M/s')
+                    reactions.append(rxn)
+                    rxn = PepperReaction([wG], [w, uG], rtype = 'side-utbr')
+                    rxn.rate_constant = (krf, '/s')
+                    reactions.append(rxn)
+                    #except DSDDuplicationError as err:
+                    #    pass
 
             elif utbr:
                 count+=1
                 a = PepperDomain('dummy_{}'.format(count), dtype='long')
                 a = PepperComplex(sequence=[a], structure=['.'], prefix='s')
                 complexes[a.name] = a
-                reactions.append(PepperReaction([w, g], [a], rtype='side-utbr', rate=kf))
-                reactions.append(PepperReaction([a], [w, g], rtype='side-utbr', rate=krf))
+                rxn = PepperReaction([w, g], [a], rtype = 'side-utbr')
+                rxn.rate_constant = (kf, '/M/s')
+                reactions.append(rxn)
+                rxn = PepperReaction([a], [w, g], rtype = 'side-utbr')
+                rxn.rate_constant = (krf, '/s')
+                reactions.append(rxn)
 
         for g in thlds:
             # T2_5
             if 'T{}_{}'.format(wi, wo) == str(g):
                 tc = complexes['Waste{}_{}'.format(wi, wo)]
                 tw = complexes['Waste_'+wo]
-                reactions.append(PepperReaction([w, g], [tc, tw], rtype='seesaw-thld', rate=kf))
+                rxn = PepperReaction([w, g], [tc, tw], rtype = 'seesaw-thld')
+                rxn.rate_constant = (kf, '/M/s')
+                reactions.append(rxn)
                 continue
 
             if utbr and global_species:
                 # TODO: check if that makes sense...?
-                if w.concentration and w.concentration.value != 0:
-                    try: # all Wires together
-                        gWn = g.name + '_W'
-                        gW = PepperDomain('dummy_{}'.format(gWn), dtype='long')
-                        gW = PepperComplex(sequence=[gW], structure=['.'], name=gWn)
-                        gW.concentration = ('initial', 0, 'M')
-                        complexes[gWn] = gW
+                if w.concentration and w.concentration[1] != 0:
+                    #try: # all Wires together
+                    gWn = g.name + '_W'
+                    gW = PepperDomain('dummy_{}'.format(gWn), dtype='long')
+                    gW = PepperComplex(sequence=[gW], structure=['.'], name=gWn)
+                    gW.concentration = ('initial', 0, 'M')
+                    complexes[gWn] = gW
 
-                        reactions.append(PepperReaction([g, uW], [gW], rtype='side-utbr', rate=kf))
-                        reactions.append(PepperReaction([gW], [g, uW], rtype='side-utbr', rate=krs))
-                    except DSDDuplicationError as err:
-                        pass
+                    rxn = PepperReaction([g, uW], [gW], rtype = 'side-utbr')
+                    rxn.rate_constant = (kf, '/M/s')
+                    reactions.append(rxn)
+                    rxn = PepperReaction([gW], [g, uW], rtype = 'side-utbr')
+                    rxn.rate_constant = (krs, '/s')
+                    reactions.append(rxn)
+                    #except DSDDuplicationError as err:
+                    #    pass
                 else:
-                    try: # all Gates together
-                        wGn = w.name + '_TH'
-                        wG = PepperDomain('dummy_{}'.format(wGn), dtype='long')
-                        wG = PepperComplex(sequence=[wG], structure=['.'], name=wGn)
-                        wG.concentration = ('initial', 0, 'M')
-                        complexes[wGn] = wG
+                    #try: # all Gates together
+                    wGn = w.name + '_TH'
+                    wG = PepperDomain('dummy_{}'.format(wGn), dtype='long')
+                    wG = PepperComplex(sequence=[wG], structure=['.'], name=wGn)
+                    wG.concentration = ('initial', 0, 'M')
+                    complexes[wGn] = wG
 
-                        reactions.append(PepperReaction([w, uT], [wG], rtype='side-utbr', rate=kf))
-                        reactions.append(PepperReaction([wG], [w, uT], rtype='side-utbr', rate=krs))
-                    except DSDDuplicationError as err:
-                        pass
+                    rxn = PepperReaction([w, uT], [wG], rtype = 'side-utbr')
+                    rxn.rate_constant = (kf, '/M/s')
+                    reactions.append(rxn)
+                    rxn = PepperReaction([wG], [w, uT], rtype = 'side-utbr')
+                    rxn.rate_constant = (krs, '/s')
+                    reactions.append(rxn)
+                    #except DSDDuplicationError as err:
+                    #    pass
 
             elif utbr:
                 count+=1
                 a = PepperDomain('dummy_{}'.format(count), dtype='long')
                 a = PepperComplex(sequence=[a], structure=['.'], prefix='s')
                 complexes[a.name] = a
-                reactions.append(PepperReaction([w, g], [a], rtype='side-utbr', rate=kf))
-                reactions.append(PepperReaction([a], [w, g], rtype='side-utbr', rate=krs))
+                rxn = PepperReaction([w, g], [a], rtype = 'side-utbr')
+                rxn.rate_constant = (kf, '/M/s')
+                reactions.append(rxn)
+                rxn = PepperReaction([a], [w, g], rtype = 'side-utbr')
+                rxn.rate_constant = (krs, '/s')
+                reactions.append(rxn)
 
     return reactions
